# #################################################################
# Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
# * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
# * Neither the name of Nokia Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.#
#
# magicnumbers.py
#
# Checks : Use of magic numbers.
#
# Reason : Magic numbers - that is, numbers that are hard-coded 
# into the source code and instead of being presented as constants 
# - make code difficult to maintain and give no indication of why 
# a calculation is the way it is. Magic numbers should be replaced 
# with named constants.
#
# #################################################################

script = CScript("magicnumbers")
script.iReString = "(.*[^a-zA-Z0-9_])([0-9]+)"
script.iFileExts = ["cpp"]
script.iCategory = KCategoryCodingStandards
script.iIgnore = KIgnoreCommentsAndQuotes
script.iSeverity = KSeverityLow

reEnum = re.compile("^\s*E\w+\s*=\s*");

def magiccompare(lines, currentline, rematch, filename):
	m = rematch.search(lines[currentline])
	if m:
		pretext = m.group(1)
		number = m.group(2)
		if (number == "0") or (number == "1") or (number == '2'):
			return 0
		elif (pretext.find("const") <> -1):
			return 0
		else:
			m2 = reEnum.search(pretext)
			if m2:
				return 0
			else:
				return 1

script.iCompare = magiccompare
scanner.AddScript(script)
